import base64
import crypto
import json
import socket
import sys

PORT = 26858

class Connection(object):
    def __init__(self, host):
        self.host = host
        self.s = None
        self.uid = None
        self.pkey = None

    def connect(self):
        try:
            self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.s.connect((self.host, PORT))
        except socket.error:
            return False
        if self.s is None:
            return False
        self.sf = self.s.makefile('r+')
        return True

    def set_key(self, user_id, pkey):
        self.uid = user_id
        self.pkey = pkey

    def cmd(self, data):
        if type(data) != dict:
            return False

        try:
            encoded = json.dumps(data)
        except ValueError:
            print 'Error: could not encode JSON'
            return False
        
        if self.pkey:
            signature = json.dumps((self.uid, base64.b64encode(crypto.rsa_sign(self.pkey, encoded))))
        else:
            signature = ''
        signature = base64.b64encode(signature)

        self.s.send(encoded + ' ' + signature + '\n')

        data = self.sf.readline()
        try:
            decoded = json.loads(data)
            if type(decoded) == dict and 'error' in decoded.keys():
                print 'Error: ' + decoded['error']
                print 'Disconnected by the server'
                self.close()
                sys.exit(0)
                return False
            return decoded
        except ValueError:
            print 'Server returned an invalid response.  Exiting...'
            return False

    def close(self):
        self.s.close()
        return True
