from Crypto.PublicKey import RSA
from Crypto.Cipher import AES
from Crypto import Random
from Crypto.Hash import SHA256
from Crypto.Signature import PKCS1_v1_5
from base64 import b64decode

def rsa_generate():
    random_gen = Random.new().read
    key = RSA.generate(1024, random_gen)
    return {'pubKey': key.publickey().exportKey('PEM'),
            'priKey': key.exportKey('PEM')}

def rsa_decrypt(priKey, enc_data):
    key = RSA.importKey(priKey)
    return key.decrypt(enc_data)

def rsa_encrypt(pubkey, plain_data):
    key = RSA.importKey(pubkey)
    return key.encrypt(plain_data, 32)[0]

def rsa_sign(priKey, data):
    key = RSA.importKey(priKey)
    signer = PKCS1_v1_5.new(key)
    hashed = SHA256.new(data)
    return signer.sign(hashed)

def rsa_verify(pubKey, signature, data):
    key = RSA.importKey(pubKey)
    signer = PKCS1_v1_5.new(key)
    hashed = SHA256.new(data)
    return signer.verify(hashed, signature)

def aes_generate():
    key = Random.new().read(32)
    iv = Random.new().read(AES.block_size)
    return {'key': key, 'iv': iv}

def aes_encrypt(key, iv, data):
    cipher = AES.new(key, AES.MODE_CFB, iv)
    return cipher.encrypt(data)

def aes_decrypt(key, iv, data):
    cipher = AES.new(key, AES.MODE_CFB, iv)
    return cipher.decrypt(data)
